class SBGuidedWarShell extends GuidedWarShell config(SBMutatorFastWarshell);

var config int ProjectileSpeed;
var config string LaunchSoundName;
var config bool bDestroyedBroadcastMsg;
var config bool bDestroyedBroadcastMsgCritical;
var config string UnknownDestroyerName;
var config string DestroyMessage;
var config string LaunchMessage;
var config byte ProjectileHealth;
var config name ValidWeaponInstigators[8];
var config bool bDebug;

var string InstigatorHumanName; // for later usage, if projectile is destroyed
var bool bIsValidWeaponInstigator;

/* no changes with this:
replication {
	reliable if(Role == ROLE_Authority)
		ProjectileSpeed;
}
*/


function bool IsValidWeaponInstigator(Actor A) {
/******************************************************************************
Returns TRUE if *A* is a candidate for mutator handling.
*A* may not be NONE.
******************************************************************************/
local byte i;
local bool result;

	result = false;
	for (i = 0; i < ArrayCount(ValidWeaponInstigators); i++)
	{
		if (bDebug)
			class'SBMutatorFastWarShell'.static.DebugLog(self, "ValidWeaponInstigators[" $ i $ "]=" $ ValidWeaponInstigators[i]);
		if (ValidWeaponInstigators[i] != '')
			if (A.IsA(ValidWeaponInstigators[i]))
			{
				result = true;
				break;
			}
	}
	if (bDebug)
		class'SBMutatorFastWarShell'.static.DebugLog(self, "IsValidWeaponInstigator returns" @ result @ "for" @ A);
	return result;
}



simulated function PostBeginPlay() {
local SBMutatorFastWarShell M;

	if (bDebug && Role == ROLE_Authority)
	{
		class'SBMutatorFastWarShell'.static.DebugLog(self, "owner=" $ owner $ ", Instigator=" $ Instigator);
		if ( ! class'SBMutatorFastWarShell'.static.FindMyMutator(self, M))
			warn(self $ ".PostBeginPlay DEBUG: my Mutator NOT found");
		else
			if ( ! M.bSBGuidedWarShellINIShown)
			{
				class'SBMutatorFastWarShell'.static.DebugLogProjectileINIValues(self);
				M.bSBGuidedWarShellINIShown = true;
			}
		class'SBMutatorFastWarShell'.static.DebugLog(self, "owner=" $ owner $ ", Instigator=" $ Instigator);
		class'SBMutatorFastWarShell'.static.DebugLog(self, "PostBeginPlay: Speed before super.PostBeginPlay():" @ speed);
	}
	Super.PostBeginPlay();
	if (bDebug)
		class'SBMutatorFastWarShell'.static.DebugLog(self, "PostBeginPlay: Speed after super.PostBeginPlay():" @ speed);
	if (Instigator != None && IsValidWeaponInstigator(Instigator))
	{
		bIsValidWeaponInstigator = true;
		InstigatorHumanName = Instigator.GetHumanName();
		Speed = class'SBMutatorFastWarShell'.static.GetSpeed(ProjectileSpeed, class'GuidedWarShell'.default.Speed, string(self.name));
		// MaxSpeed not needed here, because this projectile is not accelerated
		if (Speed > MaxSpeed)
			MaxSpeed = Speed;
		if (bDebug)
			class'SBMutatorFastWarShell'.static.DebugLog(self, "PostBeginPlay: Speed after setting ProjectileSpeed:" @ speed);
		class'SBMutatorFastWarShell'.static.ProjectileLaunched(self, LaunchSoundName, LaunchMessage, Role);
	}
}


singular function TakeDamage(int NDamage, Pawn instigatedBy, Vector hitlocation, vector momentum, name damageType) {

	class'SBMutatorFastWarShell'.static.ProjectileTakeDamage(self, ProjectileHealth, bIsValidWeaponInstigator, InstigatorHumanName, NDamage, instigatedBy, UnknownDestroyerName, DestroyMessage, hitlocation, momentum, damageType, bDestroyedBroadcastMsg, bDestroyedBroadcastMsgCritical, bDebug);
}


simulated function MoveRocket(float DeltaTime, vector CurrentVelocity, rotator GuideRotation ) {
// Copied all code from parent class, because the speed (550) is hardcoded there in the middle of code...
local int OldRoll, RollMag;
local rotator NewRot;
local float SmoothRoll;
local vector OldVelocity, X,Y,Z;

	//if (Level.NetMode == NM_Client)
	//	log(self $ ".MoveRocket DEBUG: speed=" $ speed);
	if ( (Role == ROLE_Authority) && ( (Guider == None) || (Guider.Health <= 0)
				|| (Guider.IsA('PlayerPawn') && (PlayerPawn(Guider).ViewTarget != self)) || Guider.IsInState('FeigningDeath')) )
	{
		Explode(Location,Vect(0,0,1));
		return;
	}

	ServerUpdate = Level.TimeSeconds;
	OldRoll = Rotation.Roll & 65535;
	OldVelocity = CurrentVelocity;
	//SB: adjust the Rotation:
	Velocity = CurrentVelocity + Vector(GuideRotation) * 1500 * DeltaTime;
	//Velocity = CurrentVelocity + Vector(GuideRotation) * speed * 2.72727 * DeltaTime; // 2.72727 = 1500 / 550
	// SB: now replace the hardcoded speed value of *550* with *speed*
	// Velocity = Normal(Velocity) * 550;
	Velocity = Normal(Velocity) * speed;
	NewRot = Rotator(Velocity);

	// Roll Warhead based on acceleration
	GetAxes(NewRot, X,Y,Z);
	RollMag = int(10 * (Y Dot (Velocity - OldVelocity))/DeltaTime);
	if ( RollMag > 0 )
		NewRot.Roll = Min(12000, RollMag);
	else
		NewRot.Roll = Max(53535, 65536 + RollMag);

	//smoothly change rotation
	if (NewRot.Roll > 32768)
	{
		if (OldRoll < 32768)
			OldRoll += 65536;
	}
	else if (OldRoll > 32768)
		OldRoll -= 65536;

	SmoothRoll = FMin(1.0, 5.0 * deltaTime);
	NewRot.Roll = NewRot.Roll * SmoothRoll + OldRoll * (1 - SmoothRoll);
	SetRotation(NewRot);

	if ( (Level.NetMode != NM_Standalone)
		&& ((Level.NetMode != NM_ListenServer) || (Instigator == None)
			|| (Instigator.IsA('PlayerPawn') && (PlayerPawn(Instigator).Player != None)
				&& (ViewPort(PlayerPawn(Instigator).Player) == None))) )
		AutonomousPhysics(DeltaTime);

	if ( Role == ROLE_Authority )
	{
		RealLocation = Location;
		RealVelocity = Velocity;
	}
}



defaultproperties {
	bDebug=false
	ProjectileSpeed=600
	LaunchSoundName='SBIncomingSndsV0.NuclearLaunchDetected'
	bDestroyedBroadcastMsg=false
	bDestroyedBroadcastMsgCritical=true
	UnknownDestroyerName="Someone"
	DestroyMessage="%o destroyed %k's guided Redeemer"
	LaunchMessage="%k started guiding a Redeemer"
	ProjectileHealth=5
	ValidWeaponInstigators(0)="PlayerPawn"
	ValidWeaponInstigators(1)="Bots"
	ValidWeaponInstigators(2)="Bot"

}
